
metadata = {
    'sc1a': {"true_synapse_id": "syn1876068"},
}


import os
from dreamtools import dreampath
d8c3path = os.sep.join([dreampath, 'dream8', 'D8C3'])


#gold standard prediction: https://www.synapse.org/#!Synapse:syn1902669 24Mb

